#!/usr/bin/perl
use v5.18;

use Data::Dumper    qw(Dumper);
use Storable        qw(nstore_fd dclone);
use Sereal::Encoder qw(encode_sereal);
use JSON            qw(encode_json);

my $stores = {
    dumper   => sub { Dumper( $_[0] ) },
    jsoner   => sub { encode_json( $_[0] ) },
    serealer => sub { encode_sereal( $_[0] ) },
    storer   => sub {
        open my $sfh, '>:raw', \ my $string;
        nstore_fd( $_[0], $sfh );
        close $sfh;
        $string;
        },
    };

my $max_hash_count = 10;

my $hash;
my @keys   = get_keys();
my @values = get_values();
@{$hash}{ @keys } = @values;

my %lengths;
my @max;

foreach my $hash_count ( 1 .. $max_hash_count ) {
    my $data = [ map { dclone $hash } 1 .. $hash_count ];

    foreach my $type ( sort keys %$stores ) {
        my $string = $stores->{$type}( $data );
        my $length = length $string;

        $max[$hash_count] = $length if $length > $max[$hash_count];
        $max[0] = $length if $length > $max[0]; # grand max
        if( 0 == $length ) {
            warn "$type: length is zero!\n";
            }
        push @{$lengths{$type}}, $length;
        }
    }

###########
# make a tab separated report with the normalized numbers
# for each method, in columns suitable for a spreadsheet
say join "\t", sort keys %$stores;
open my $per_fh, '>:utf8', "$0-per.tsv" or die "$!";
open my $grand_fh, '>:utf8', "$0-grand.tsv" or die "$!";

foreach my $index ( 1 .. $max_hash_count ) {
    say { $per_fh } join(
            "\t",
            map { $lengths{$_}[$index - 1] / $max[$index] }
            sort keys %$stores
        );
    say { $grand_fh } join(
            "\t",
            map { $lengths{$_}[$index - 1] / $max[0] }
            sort keys %$stores
        );
    }

# make some long keys
sub get_keys {
    map { $0 . time() . $_ . $$ } ( 'a' .. 'f' );
    }

# make some long values
sub get_values {
    map { $0 . time() . $_ . $$ } ( 'f' .. 'k' );
    }